import _extends from "@babel/runtime/helpers/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/objectWithoutPropertiesLoose";
import React from 'react';
import PropTypes from 'prop-types';
import { reactComponentPropType } from './propTypesValidators';

var RequestContainer = function RequestContainer(_ref) {
  var request = _ref.request,
      children = _ref.children,
      Component = _ref.component,
      isDataEmpty = _ref.isDataEmpty,
      showLoaderDuringRefetch = _ref.showLoaderDuringRefetch,
      noDataMessage = _ref.noDataMessage,
      ErrorComponent = _ref.errorComponent,
      errorComponentProps = _ref.errorComponentProps,
      LoadingComponent = _ref.loadingComponent,
      loadingComponentProps = _ref.loadingComponentProps,
      extraProps = _objectWithoutPropertiesLoose(_ref, ["request", "children", "component", "isDataEmpty", "showLoaderDuringRefetch", "noDataMessage", "errorComponent", "errorComponentProps", "loadingComponent", "loadingComponentProps"]);

  var dataEmpty = isDataEmpty(request);

  if (request.pending > 0 && (showLoaderDuringRefetch || dataEmpty)) {
    return LoadingComponent ? React.createElement(LoadingComponent, loadingComponentProps) : null;
  }

  if (request.error) {
    return ErrorComponent ? React.createElement(ErrorComponent, _extends({
      error: request.error
    }, errorComponentProps)) : null;
  }

  if (dataEmpty) {
    return noDataMessage;
  }

  if (children) {
    return typeof children === 'function' ? children(request) : children;
  }

  return React.createElement(Component, _extends({
    request: request
  }, extraProps));
};

RequestContainer.defaultProps = {
  isDataEmpty: function isDataEmpty(request) {
    return Array.isArray(request.data) ? request.data.length === 0 : !request.data;
  },
  showLoaderDuringRefetch: true,
  noDataMessage: null
};
RequestContainer.propTypes = process.env.NODE_ENV !== "production" ? {
  children: PropTypes.oneOfType([PropTypes.func, PropTypes.node]),
  component: reactComponentPropType('RequestContainer'),
  request: PropTypes.shape({
    data: PropTypes.any,
    error: PropTypes.any,
    pending: PropTypes.number.isRequired
  }).isRequired,
  isDataEmpty: PropTypes.func,
  showLoaderDuringRefetch: PropTypes.bool,
  noDataMessage: PropTypes.node,
  errorComponent: reactComponentPropType('RequestContainer'),
  errorComponentProps: PropTypes.objectOf(PropTypes.any),
  loadingComponent: reactComponentPropType('RequestContainer'),
  loadingComponentProps: PropTypes.objectOf(PropTypes.any)
} : {};
export default RequestContainer;